﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Transactions;

namespace WCFTransactions
{
	public class DistributedStoreClient : IDisposable
	{
		List<ChannelFactory<IDistributedStoreContract>> _factories;
		List<IDistributedStoreContract> _channels;
		bool _disposed;

		public DistributedStoreClient(params int[] ports)
		{
			if(ports == null || ports.Length == 0)
				throw new ArgumentNullException("ports");

			_factories = new List<ChannelFactory<IDistributedStoreContract>>();
			_channels = new List<IDistributedStoreContract>();

			Binding binding = Bindings.CreateTcpBinding();

			foreach(int port in ports)
			{
				ChannelFactory<IDistributedStoreContract> factory = new ChannelFactory<IDistributedStoreContract>(
					 binding,
					 new EndpointAddress("net.tcp://localhost:" + port + "/membership"));

				//factory.Endpoint.Behaviors.Add(new ConsoleMessageTracing());

				IDistributedStoreContract channel = factory.CreateChannel();

				_factories.Add(factory);
				_channels.Add(channel);
			}
		}

		public void Dispose()
		{
			if(!_disposed)
			{
				foreach(IDistributedStoreContract channel in _channels)
					((IClientChannel)channel).Dispose();

				foreach(ChannelFactory<IDistributedStoreContract> factory in _factories)
					factory.Close();

				_disposed = true;
			}
		}

		public bool Save(string data)
		{
			bool savedOnAllNodes = true;

			using(TransactionScope scope = new TransactionScope())
			{
				foreach(IDistributedStoreContract channel in _channels)
				{
					if(!channel.SaveOperation(data))
					{
						savedOnAllNodes = false;
						//break;
					}
				}

				if(savedOnAllNodes)
					scope.Complete();
			}

			return savedOnAllNodes;
		}
	}
}
