﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Transactions;

namespace WCFTransactions
{
	[ServiceBehavior(
		ConcurrencyMode = ConcurrencyMode.Single,
		InstanceContextMode = InstanceContextMode.Single,
		ReleaseServiceInstanceOnTransactionComplete = false)]
	public class DistributedStoreService : IDistributedStoreContract, IDisposable
	{
		TransactionalList<string> _nodes;
		int _port;
		ServiceHost _serviceHost;
		bool _disposed;

		public DistributedStoreService(int port)
		{
			_nodes = new TransactionalList<string>();
			_nodes.Port = port;
			_nodes.Trace = true;

			_port = port;

			Binding binding = Bindings.CreateTcpBinding();

			_serviceHost = new ServiceHost(this);

			_serviceHost.AddServiceEndpoint(typeof(IDistributedStoreContract),
				binding,
				new Uri("net.tcp://localhost:" + port + "/membership"));

			_serviceHost.Open();
		}

		public void Dispose()
		{
			if(!_disposed)
			{
				if(_serviceHost != null)
					_serviceHost.Close();

				_disposed = true;
			}
		}

		[OperationBehavior(TransactionScopeRequired = true, TransactionAutoComplete = true)]
		bool IDistributedStoreContract.SaveOperation(string data)
		{
			Console.WriteLine("Operation Save called at port {0}: {1}", _port, data);

			// Define the probability randomly of failing
			bool storageFailed = new Random().Next(1, 100) < 85;

			if(storageFailed)
				_nodes.Add(data);

			Console.Write("  -> ");
			foreach(string cn in _nodes.Value)
				Console.Write("{0} ", cn);

			if(!storageFailed)
				Console.Write("ERROR");

			Console.WriteLine();

			return storageFailed;
		}
	}
}
